import { Model } from "../../model";
import { DOMView } from "../../core/dom_view";
export class AbstractIconView extends DOMView {
}
AbstractIconView.__name__ = "AbstractIconView";
export class AbstractIcon extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
AbstractIcon.__name__ = "AbstractIcon";
//# sourceMappingURL=abstract_icon.js.map