export const root: string
export const tool_icon_copy_to_clipboard: string
export const tool_icon_replace_mode: string
export const tool_icon_append_mode: string
export const tool_icon_intersect_mode: string
export const tool_icon_subtract_mode: string
export const tool_icon_clear_selection: string
export const tool_icon_box_select: string
export const tool_icon_box_zoom: string
export const tool_icon_zoom_in: string
export const tool_icon_zoom_out: string
export const tool_icon_help: string
export const tool_icon_hover: string
export const tool_icon_crosshair: string
export const tool_icon_lasso_select: string
export const tool_icon_pan: string
export const tool_icon_xpan: string
export const tool_icon_ypan: string
export const tool_icon_range: string
export const tool_icon_polygon_select: string
export const tool_icon_redo: string
export const tool_icon_reset: string
export const tool_icon_save: string
export const tool_icon_tap_select: string
export const tool_icon_undo: string
export const tool_icon_wheel_pan: string
export const tool_icon_wheel_zoom: string
export const tool_icon_box_edit: string
export const tool_icon_freehand_draw: string
export const tool_icon_poly_draw: string
export const tool_icon_point_draw: string
export const tool_icon_poly_edit: string
export const tool_icon_line_edit: string
export default ""
